#include <stdlib.h>

#ifdef USE_NLLIBC
#include <nllibc.h>
#endif

#include "type.h"

static struct word {
  c_type_t type;
  char *word;
} words[] = {
  { C_TYPE_NONE,         "NONE" },

  { C_TYPE_WORD,         "WORD" },
  { C_TYPE_WORDS,        "WORDS" },

  { C_TYPE_NUMBER,       "NUMBER" },
  { C_TYPE_STRING,       "STRING" },
  { C_TYPE_SYMBOL,       "SYMBOL" },
  { C_TYPE_ARRAY,        "ARRAY" },
  { C_TYPE_VALUE,        "VALUE" },
  { C_TYPE_VARIABLE,     "VARIABLE" },
  { C_TYPE_FUNCTION,     "FUNCTION" },
  { C_TYPE_LABEL,        "LABEL" },
  { C_TYPE_BLOCK,        "BLOCK" },
  { C_TYPE_STACK,        "STACK" },
  { C_TYPE_NOP,          "NOP" },

  { C_TYPE_ADDRESS,      "ADDRESS" },
  { C_TYPE_POINTER,      "POINTER" },
  { C_TYPE_PLUS,         "PLUS" },
  { C_TYPE_MINUS,        "MINUS" },
  { C_TYPE_CAST,         "CAST" },
  { C_TYPE_ARG,          "ARG" },

  { C_TYPE_COMMENT_IN,   "/*" },
  { C_TYPE_COMMENT_OUT,  "*/" },
  { C_TYPE_SQUOAT,       "\'" },
  { C_TYPE_DQUOAT,       "\"" },

  { C_TYPE_IF,           "if" },
  { C_TYPE_ELSE,         "else" },
  { C_TYPE_FOR,          "for" },
  { C_TYPE_WHILE,        "while" },
  { C_TYPE_DO,           "do" },
  { C_TYPE_SWITCH,       "switch" },
  { C_TYPE_CASE,         "case" },
  { C_TYPE_DEFAULT,      "default" },
  { C_TYPE_GOTO,         "goto" },
  { C_TYPE_BREAK,        "break" },
  { C_TYPE_CONTINUE,     "continue" },
  { C_TYPE_RETURN,       "return" },

  { C_TYPE_EXTERN,       "extern" },
  { C_TYPE_STATIC,       "static" },

  { C_TYPE_SIGNED,       "signed" },
  { C_TYPE_UNSIGNED,     "unsigned" },
  { C_TYPE_SHORT,        "short" },
  { C_TYPE_LONG,         "long" },
  { C_TYPE_CHAR,         "char" },
  { C_TYPE_INT,          "int" },
  { C_TYPE_FLOAT,        "float" },
  { C_TYPE_DOUBLE,       "double" },
  { C_TYPE_VOID,         "void" },
  { C_TYPE_STRUCT,       "struct" },
  { C_TYPE_UNION,        "union" },
  { C_TYPE_ENUM,         "enum" },
  { C_TYPE_TYPEDEF,      "typedef" },

  { C_TYPE_SIZEOF,       "sizeof" },
  { C_TYPE_RSHIFTEQ,     ">>=" },
  { C_TYPE_LSHIFTEQ,     "<<=" },

  { C_TYPE_ARROW,        "->" },
  { C_TYPE_INC,          "++" },
  { C_TYPE_DEC,          "--" },
  { C_TYPE_ADDEQ,        "+=" },
  { C_TYPE_SUBEQ,        "-=" },
  { C_TYPE_MULEQ,        "*=" },
  { C_TYPE_DIVEQ,        "/=" },
  { C_TYPE_MODEQ,        "%=" },
  { C_TYPE_ANDEQ,        "&=" },
  { C_TYPE_XOREQ,        "^=" },
  { C_TYPE_OREQ,         "|=" },
  { C_TYPE_RSHIFT,       ">>" },
  { C_TYPE_LSHIFT,       "<<" },
  { C_TYPE_GTEQ,         ">=" },
  { C_TYPE_LTEQ,         "<=" },
  { C_TYPE_EQEQ,         "==" },
  { C_TYPE_NEQ,          "!=" },
  { C_TYPE_ANDAND,       "&&" },
  { C_TYPE_OROR,         "||" },

  { C_TYPE_ADD,          "+" },
  { C_TYPE_SUB,          "-" },
  { C_TYPE_MUL,          "*" },
  { C_TYPE_DIV,          "/" },
  { C_TYPE_MOD,          "%" },
  { C_TYPE_AND,          "&" },
  { C_TYPE_XOR,          "^" },
  { C_TYPE_OR,           "|" },
  { C_TYPE_INV,          "~" },
  { C_TYPE_GT,           ">" },
  { C_TYPE_LT,           "<" },
  { C_TYPE_EQ,           "=" },
  { C_TYPE_NOT,          "!" },
  { C_TYPE_DOT,          "." },
  { C_TYPE_COMMA,        "," },
  { C_TYPE_QUESTION,     "?" },

  { C_TYPE_COLON,        ":" },
  { C_TYPE_SEMICOLON,    ";" },
  { C_TYPE_BRACKET_IN,   "(" },
  { C_TYPE_BRACKET_OUT,  ")" },
  { C_TYPE_MBRACKET_IN,  "{" },
  { C_TYPE_MBRACKET_OUT, "}" },
  { C_TYPE_BBRACKET_IN,  "[" },
  { C_TYPE_BBRACKET_OUT, "]" },

  { C_TYPE_NONE,          NULL }
};

/*****************************************************************
 * library
 */

char *type_get_word(c_type_t type)
{
  int i;
  for (i = 0; words[i].word; i++) {
    if (words[i].type == type)
      return words[i].word;
  }
  return NULL;
}
